<?php
class Ship2Anywhere_Shipping_Rules extends WC_Shipping_Method
{
    public function __construct()
    {
        $this->id = SHIP2ANYWHERE_RULES;
        $this->title = __(SHIP2ANYWHERE_COMPANY_NAME, 'ship2anywhere');
        $this->method_title = __(SHIP2ANYWHERE_COMPANY_NAME, 'ship2anywhere');
        $this->method_description = __('Multi Carrier Shipping and Logistics Technology able to seamlessly integrate into your Woo cart.', 'ship2anywhere');

        $this->form_fields = include 'ship2anywhere-config.php';
        $this->init_settings();
        $this->define_woocommerce_hooks();
    }

    private function define_woocommerce_hooks()
    {
        add_action('woocommerce_update_options_shipping_' . $this->id, array($this, 'process_admin_options'));
        add_filter('woocommerce_shipping_methods', [$this, 'shipping_methods'], 10, 1);
    }

    public function shipping_methods($methods)
    {
        array_push($methods, __CLASS__);
        return $methods;
    }

    public function get_shipping_origin()
    {
        return [
            'country' => $this->get_option('warehouse_country'),
            'city' => $this->get_option('wh_city'),
            'postcode' => $this->get_option('wh_postcode'),
        ];
    }

    public function get_shipping_destination($address)
    {
        return [
            'country' => $address['country'],
            'city' => $address['city'],
            'postcode' => $address['postcode'],
        ];
    }

    public function calculate_shipping($package = [])
    {
        try {
            $params = $this->generate_quotes_params($package);
            $quotes = $this->get_quotes($params);

            if ($quotes) {
                foreach ($quotes as $q) {
                    $this->add_rate($q);
                }
            }
        } catch (Exception $e) {
            Ship2Anywhere_Utility::logger($result, 'CALCULATE SHIPPING ERROR', 'ship2anywhere-calculate-shipping-error');
        }
    }

    public function get_default_packaging_settings()
    {
        return [
            'length' => (float) $this->get_option('default_length'),
            'width' => (float) $this->get_option('default_width'),
            'height' => (float) $this->get_option('default_height'),
            'weight' => (float) $this->get_option('default_weight'),
        ];
    }

    public function generate_quotes_params($package = [])
    {
        $params = [];
        $default = $this->get_default_packaging_settings();

        $params['channel'] = (string) $this->get_option('channel');
        $params['source'] = $this->get_shipping_origin();
        $params['destination'] = $this->get_shipping_destination($package['destination']);
        $params['items'] = [];
        $params['pickupDetails'] = [
            'asap' => true,
            'pickupDate' => null,
            'pickupEarliestTime' => null,
            'pickupLatestTime' => null,
            'pickupTimezone' => $this->get_option('gmt_offset'),
        ];

        $params['items'] = $this->generate_items($package['contents'], $default);
        $params['packages'] = $this->calculate_packaging($package);

        return $params;
    }

    private function generate_items($contents, $default)
    {
        $content_data = [];
        if (!isset($_REQUEST['product-id'])) {
            foreach ($contents as $item) {
                $terms_list = wp_get_post_terms($item['product_id'], 'product_cat');
                array_push($content_data, $this->generate_item_data($item, $default));
            }
            return $content_data;
        }

        $product_id = absint($_REQUEST['product-id']);
        $product = wc_get_product($product_id);
        array_push($content_data, $this->generate_item_data($product, $default));

        return $content_data;
    }

    private function calculate_packaging($packages)
    {
        $payload = [];
        $default = $this->get_default_packaging_settings();

        foreach ($packages['contents'] as $item) {
            if ($item['data']->needs_shipping()) {
                $payload[] = [
                    'id' => null,
                    'quantity' => (int) $item['quantity'],
                    'length' => (float) $item['data']->get_length() ?: $default['length'],
                    'width' => (float) $item['data']->get_width() ?: $default['width'],
                    'height' => (float) $item['data']->get_height() ?: $default['height'],
                    'weight' => (float) $item['data']->get_weight() ?: $default['weight'],
                    'weightUnit' => $this->get_option('default_weight_unit'),
                    'measureUnit' => $this->get_option('default_measure_unit'),
                ];
            }
        }

        if (count($payload) < 1) {return [];}

        $apiInstance = new Ship2Anywhere_Api();
        $response = $apiInstance->send_api_request(
            'POST',
            'packaging/packaging/calculate',
            [],
            $payload
        );

        if (is_null($response)) return [];

        $selectedPackaging = $response[0];
        $packaging = [];
        foreach ($selectedPackaging->packaging as $pkg) {
            $packaging[] = [
                'length' => (float) $pkg->length,
                'width' => (float) $pkg->width,
                'height' => (float) $pkg->height,
                'weight' => (float) $pkg->maxWeight,
                'quantity' => (int) $pkg->qty,
                'weightUnit' => $pkg->weightUnit,
                'measureUnit' => $pkg->measureUnit,
                'contentWeight' => (float) $pkg->contentWeight,
            ];
        }

        return $packaging;
    }

    private function generate_item_data($item, $default)
    {
        return [
            'quantity' => (int) $item['quantity'],
            'value' => (int) $item['data']->get_regular_price(),
            'length' => (float) $item['data']->get_length() ?: $default['length'],
            'width' => (float) $item['data']->get_width() ?: $default['width'],
            'height' => (float) $item['data']->get_height() ?: $default['height'],
            'weight' => (float) $item['data']->get_weight() ?: $default['weight'],

            "code" => "TST",
            'weightUnit' => $this->get_option('default_weight_unit'),
            'measureUnit' => $this->get_option('default_measure_unit'),
            'currency' => $this->get_option('default_currency'),
        ];
    }

    public function get_quotes($params = [])
    {
        if (Ship2Anywhere_Utility::get_config('enabled') == 'yes') {
            $apiInstance = new Ship2Anywhere_Api();
            $response = $apiInstance->send_api_request(
                'POST', 
                'carrier/quote', 
                [], 
            $params);
            Ship2Anywhere_Utility::logger($response, 'get quotes response', 'get-quotes-response', true);
            return $this->parse_quotes($response);
        }
    }

    public function parse_quotes($response)
    {
        $rates = [];
        if (is_array($response) || is_object($response)) {
            if (empty($response)) {return $rates;}

            usort($response, function($a, $b) {
                return ($a->amount < $b->amount) ? -1 : 1;
            });

            foreach ($response as $carrier) {
                if (is_string($carrier) || is_null($carrier)) {
                    continue;
                }

                $carrier_profile_id = $carrier->carrierProfileId;
                $carrier_name = $carrier->carrierName;
                $currency = $carrier->currency;
                $eta = $carrier->eta;
                $amount = $carrier->amount;
                $tax = $carrier->tax;

                $service_profile_id = $carrier->serviceProfileId;
                $service_profile_name = $carrier->serviceProfileName;
                $label = $carrier_name;

                if ($carrier->operation) {
                    $label = $label . '[' . $carrier->operation->name . ']';
                }

                $label = $label . ' [' . $service_profile_name . '] [' . $eta . '] [Tax: ' . $currency . ' $' . $tax . ' ]';

                array_push($rates, [
                    'id' => $carrier_profile_id . '_' . $service_profile_id,
                    'label' => $label,
                    'cost' => $amount,
                    'meta_data' => array(
                        'shipping_method' => $carrier_profile_id . '_' . $service_profile_id,
                    ),
                ]);
            }
        }

        return $rates;
    }
}
